/*!
 * 💡 - 求解最佳悬浮位置
 * https://github.com/hai2007/tool.js/blob/master/calcHoverPosition.js
 *
 * author hai2007 < https://hai2007.gitee.io/sweethome >
 *
 * Copyright (c) 2022-present hai2007 走一步，再走一步。
 * Released under the MIT license
 */

export default function (points, hoverSize, regionSize) {

    var hoverPosition = [];
    var degs = [0, -0.25, -0.5, -0.625, -0.75, -0.875, -1, -1.125, -1.25, -1.375, -1.5, -1.675, -1.75, -1.875];

    var index, flag, k, deg, dis;
    for (index = 0; index < points.length; index++) {
        hoverPosition[index] = [
            points[index][0] - hoverSize[0] * 0.5,
            points[index][1] - hoverSize[1] - 10,
        ];

        deg = 0;
        dis = 10 + hoverSize[1] * 0.5;

        flag = 0;
        while (true) {
            // 如果超过计算次数上限，放弃
            if (flag > 10000) {
                hoverPosition[index] = [
                    points[index][0] - hoverSize[0] * 0.5,
                    points[index][1] - hoverSize[1] - 10,
                ];
                break;
            }

            //  校对参数
            if (deg >= degs.length) {
                deg = 0;
                dis += 2;
            }

            // 更新新的位置
            var cos = Math.cos(degs[deg] * Math.PI),
                sin = Math.sin(degs[deg] * Math.PI);
            var hoverCenter = [
                (points[index][0] - points[index][0]) * cos -
                (points[index][1] - dis - points[index][1]) * sin +
                points[index][0],
                (points[index][0] - points[index][0]) * sin +
                (points[index][1] - dis - points[index][1]) * cos +
                points[index][1],
            ];

            hoverPosition[index] = [
                hoverCenter[0] - hoverSize[0] * 0.5,
                hoverCenter[1] - hoverSize[1] * 0.5,
            ];

            deg += 1;

            flag += 1;

            // 判断是否越界
            if (
                hoverPosition[index][0] < 0 ||
                hoverPosition[index][0] + hoverSize[0] > regionSize[0] ||
                hoverPosition[index][1] < 0 ||
                hoverPosition[index][1] + hoverSize[1] > regionSize[1]
            ) {
                continue;
            }

            // 判断和已经有的是不是会重叠
            var doubleFlag = false;
            for (k = 0; k < index; k++) {
                if (
                    // x重叠
                    (hoverPosition[index][0] > hoverPosition[k][0]
                        ? hoverPosition[index][0] < hoverPosition[k][0] + hoverSize[0] + 5
                        : hoverPosition[k][0] <
                        hoverPosition[index][0] + hoverSize[0] + 5) &&
                    // y重叠
                    (hoverPosition[index][1] > hoverPosition[k][1]
                        ? hoverPosition[index][1] < hoverPosition[k][1] + hoverSize[1] + 5
                        : hoverPosition[k][1] < hoverPosition[index][1] + hoverSize[1] + 5)
                ) {
                    doubleFlag = true;
                    break;
                }
            }
            if (doubleFlag) {
                continue;
            }

            var pointFlag = false;
            // 判断是否和已经有的点重叠
            for (k = 0; k < points.length; k++) {
                if (
                    points[k][0] + 10 > hoverPosition[index][0]
                    && points[k][0] - 10 < hoverPosition[index][0] + hoverSize[0]
                    && points[k][1] + 10 > hoverPosition[index][1]
                    && points[k][1] - 10 < hoverPosition[index][1] + hoverSize[1]) {
                    pointFlag = true;
                    break;
                }
            }
            if (pointFlag) {
                continue;
            }

            // 如果上面的检查都通过了，ok，得到了一个解
            break;
        }
    }

    return hoverPosition;
};
